/*
 * Project name:
     SMS_Home_Alarm_2
 * Copyright:
     (c) Mikroelektronika, 2013.
 * Revision History:
     20131115:
       - initial release (DO);
 * Description:
     This is revised SMS Home Alarm System project
     with new parts and old funtion - catch the intruder.
     Motion detector sensor is used to detect movement in indoor perimeter.
     You can customize a level of movement which is required to trigger the alarm.
     First the SMS will be sent to predefined number and after that you have 
     few more seconds to disarm the device.
     If movement is still detectable, Siren or any other device can be activated
     with help of Relay click.
 * Test configuration:
     MCU:             PIC18F47j53
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39964B.pdf
     Dev.Board:       PIC clicker
                      http://www.mikroe.com/pic/clicker/
     Oscillator:      HS-PLL 48.0000 MHz, 16.0000 MHz Crystal
     Ext. Modules:    GSM click
                      http://www.mikroe.com/click/gsm/
                      RELAY click
                      http://www.mikroe.com/click/relay/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
 >>> Adjust the number to which the SMS will be sent and recompile the project
     now have HEX file ready for programming.
    
     A THRESHOLD value allows you to adjust the reaction time of Alarm triggering.
    
     Follow the video instruction to setup your hardware. ac:SMS_Home_Alarm_2
 */

// Configure phone number on which the SMS warning will be sent
char phoneNumber[] = "+3816xxxxxxxx";    // Use international code + your phone number
 
#include "GSM_Telit.h"
#define A_THRESHOLD 100                  // Experiment with values 20, 50, 100, etc.
// PIC clicker pin definitions
sbit LD1 at LATA0_bit;
sbit LD2 at LATA1_bit;
sbit LD1_Direction at TRISA0_bit;
sbit LD2_Direction at TRISA1_bit;

sbit T1 at RD3_bit;
sbit T2 at RD2_bit;
sbit T1_Direction at TRISD3_bit;
sbit T2_Direction at TRISD2_bit;
// PIC clicker pin definitions

// Motion Sensor definitions
sbit MotionSens at PORTE.B0;
sbit MotionSens_Direction at TRISE0_bit;
// Motion Sensor definitions

//
sbit RL1 at LATB3_bit;
sbit RL1_Direction at TRISB3_bit;
sbit Siren at LATC6_bit;
sbit Siren_Direction at TRISC6_bit;

char alarmActive = 0;
char motionDetected = 0;

void Delay_35ms(){
  Delay_ms(35);
}
// Simple LED action indicating a button press
void Blink(){
  LATA &= 0xFC;        // AND mask 1111 1100 turn OFF(only) last two pins RA0 RA1
  Delay_35ms();
  LATA |= 0x03;        // OR mask  0000 0011 turn ON (only) last two pins RA0 RA1
  Delay_35ms();
  LATA &= 0xFC;        // AND mask 1111 1100 turn OFF(only) last two pins RA0 RA1
  Delay_35ms();
}

void MCU_Init()
{
  ANCON0 = 0xFF;               // Default all pins to digital
  ANCON1 = 0x1F;               //
  // Conigure Remappable UART pins
  Unlock_IOLOCK();
  PPS_Mapping_NoLock(22, _INPUT,  _RX2_DT2);  // RX - RP22.
  PPS_Mapping_NoLock(23, _OUTPUT, _TX2_CK2);  // TX - RP23.
  Lock_IOLOCK();
  /* Configure direction and initial value for importan pins */
  // Relay click and motion sensor
  RL1 = 0;
  RL1_Direction = 0;
  Siren = 0;
  Siren_Direction = 0;
  MotionSens_Direction = 1;
  // GSM click
  RST = 0;
  RST_Direction = 0;
  PWRMON_Direction = 1;
  // PIC clicker Buttons
  T1_Direction = 1;             // Button pis are set as input
  T2_Direction = 1;             //
  // PIC clicker LEDs
  LD1 = 0;                      // Turn OFF both LEDs
  LD2 = 0;                      //
  LD1_Direction = 0;            // Set direction for LEDS
  LD2_Direction = 0;            //
  // Initialize Remappable UART module
  UART_Remappable_Init(9600);   // Using default baud rate 9600
}

void Alarm(){
int i;
  SendSMS(phoneNumber);         // First things first, send the SMS
  LD1 = 1;                      // Prepare the LEDs
  LD2 = 0;
  for(i = 0; i < 120; i++){     // Blinkg the LEDs as warning befor the Siren
    Delay_ms(77);
    LD1 = ~LD1;
    LD2 = ~LD2;
    if (T2 == 0){               // Siren can be stoped with Button2
      return;
    }
  }
  LD1 = 0;
  LD2 = 1;
  for(i = 0; i < 10; i++){      // Test few times, so you don't misfire
    if (MotionSens == 1)        // If motion is still detectable
      Siren = 1;                // Activate the Siren (Relay)
    Delay_1ms();
  }
}
// main function
void main() {
  char cnt = 50;
  
  MCU_Init();                            // Initialize MCU pins and modules
  if (GSM_Init() == 0)                   // Initialize GSM module, if AT is received LED1 is ON
    LD1 = 1;
  while(1){                              // Endless loop
    if (alarmActive){                    // First we check for Alarm active flag
      if (MotionSens == 1){              // Next we chek if motion is detected
        motionDetected++;                // Motion accumulation, trying to reach the threshold value(it's used to provide enough time for Alarm to be stoped)
        Delay_ms(45);                    // Check is executed ~20 times per second.
      }
      else{                              // If there is no detection
        if (motionDetected > 0){         //   and we have motion accumulation, *slowly reduce motion accumulation
          if (!cnt--){                   // Slowly mechanism
            motionDetected--;
            cnt = 50;                    // "Slowness" factor 1
          }
          Delay_10ms();                  // "Slowness" factor 2
        }
        else
          Delay_1sec();                  // Where's the rush, wait a little bit
      }
      if (motionDetected > A_THRESHOLD){ // Accumulation reached the threshold value
        motionDetected = 0;              // reset the accumulator
        if (Siren == 0){                 // Check if Siren is OFF
          Alarm();                       //  then activate the Alarm!
        }
      }
    }
    if (T1 == 0){                        // With Button1 we activate the Alarm
      Blink();                           // Blink to indicate a button press
      LD2 = 1;                           // Show the alaram state on LED2
      alarmActive = 1;                   // Set the flag so that motion can be cheked
      motionDetected = 0;                // Reset the accumulator
      Siren = 0;                         // Turn OFF the Siren, wait for another intrusion detection
    }
    if (T2 == 0){                        // With Button2 we can deactivate the Alarm
      Blink();                           // Blink to indicate the button press
      LD2 = 0;                           // Show the Alarm state
      alarmActive = 0;                   // Set the flag
      motionDetected = 0;                // Reset the accumulator
      Siren = 0;                         // Turn OFF the Siren, it's giving me a headache
    }
  }
}